# Environment quality and partner choice

@[toc]

## Run the simulation

### Compile the project

Download the source code, install the dependencies (boost, cmake, sdl2, sdl2-image, zlib and eigen). Then, run

```
cmake CMakeLists.txt
make
```

to compile roborobo with the wander coop setup. Internally, the project is called _negociate_.

To run the project, you need to run `python batch_negociate.py`.

`batch_negociate.py` is made to be run on a cluster. You _must_ set the variable `cluster=False` in the file to run it on your personal computer. Otherwise, `batch_negociate.py` will spawn so many processes that it can crash your computer. You must also change the output path (`logdir` variable).

run once `batch_negociate.py` without argument. It will answer back the number of run needed to do the full grid search. It is likely that the amount of runs is huge (hundreds or thousand runs). You can reduce this number by lowering the number of parameters in the `gridconf` dict or reducing the number of simulations per condition (`gridconf['_rep']`).

Then, to run all the simulations, you can do

```
nbjob=`python batch_negociate.py`
for i in {1..$nbjob}
do
    python batch_negociate.py $i
done
```

or to run only the first one

```
python batch_negociate.py 1
```

You can also directly start roborobo with your own params

```
python pyevoroborobo.py --no-movie -e fitprop -g 1500 -s 0.001 -l config/negociate.properties -- +gInitialNumberOfRobots 1000 +tau 0
```

params that start with a `+` overrides the value found in `config/negociate.properties`.

## Run the analyses

Once you have done all your runs, you must run the analyses script that will evaluate the robots behavior. Use the script `ba_negociate.sh` to run additionnal analyses.

```
./ba_negociate.sh -p [nbOfSimultaneousEvaluation] <generationToAnalyse> <path/to/your/run> [<path/to/your/run2>…]
```

For instance, to analyses all the generation n° 199 of the runs in the folder logs/ using 3 CPU, use

```
./ba_negociate.sh -p 3 199 logs/*/rep00/
```

The option -d only start the first analyses provided in debug mode, the command -f only output the commands that `ba_negociate` will start without running them.

Then, all the analyses are in the jupyter notebook `figures.ipynb` present in the analyses folder in the source code. Data used for the paper are present in the pkl files.

You can reproduce the results in these pkl files by running all the simulation with their exact same parameters.
